"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = require("fs");
var parseAst_1 = require("../parseAst");
var types_1 = require("../types");
function cliMain(o) {
    preconditions(o);
    var input = fs_1.existsSync(o.input) ? fs_1.readFileSync(o.input).toString() : o.input;
    var result = parseAst_1.parseAst(__assign({}, o, { input: input }));
    var text = o.outputStyle === 'json' ? JSON.stringify(result, null, 2) : JSON.stringify(result);
    if (o.output) {
        fs_1.writeFileSync(o.output, text);
    }
    else {
        process.stdout.write(text);
    }
}
exports.cliMain = cliMain;
function preconditions(options) {
    if (options.output) {
        options.output = options.output.split(',');
    }
    if (options.help) {
        printHelp();
        process.exit(0);
    }
    if (options.listLanguages) {
        process.stdout.write(listLanguages());
        process.exit(0);
    }
    if (!options.language || !options.input) {
        fail("--language and --input are mandatory but one was not given. Aborting.");
    }
    if (!types_1.languages.includes(options.language)) {
        fail("Unknown language \"" + options.language + "\".\n    " + listLanguages() + "\n    Aborting.");
    }
}
function listLanguages() {
    return "\nSupported Languages: \n" + types_1.languages.join(', ') + "\n";
}
function fail(s) {
    console.error(s);
    process.exit(1);
}
function printHelp() {
    console.log("\nUsage: \n\nunivac --language python3 --input src/main.py --output main.py.ast\n\nOptions:\n  ");
}
//# sourceMappingURL=cli.js.map