"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var visitor_1 = require("./visitor");
var antlr4 = require('antlr4');
function parseAst(options) {
    var input = options.input;
    var info = getParserForLanguage(options.language);
    var chars = new antlr4.InputStream(input);
    var lexer = new info.Lexer(chars);
    var tokens = new antlr4.CommonTokenStream(lexer);
    var parser = new info.Parser(tokens);
    parser.buildParseTrees = true;
    var tree = parser[info.mainRule]();
    if (tree.exception) {
        throw new Error('Parser exception: ' + tree.exception);
    }
    var visitor = new visitor_1.Visitor(options);
    tree.accept(visitor);
    var ast = visitor.getAst();
    // removeRedundantNode(ast, info)
    // removeRedundantNode(ast, info)
    return removeRedundantNode(ast, info);
}
exports.parseAst = parseAst;
function removeRedundantNode(node, info) {
    // let complete = false
    do {
    } while (removeRedundantNode_(node, info) > 0);
    return node;
}
function removeRedundantNode_(node, info, parent, count) {
    var _a;
    if (count === void 0) { count = { n: 0 }; }
    if (parent && info.redundantTypes && info.redundantTypes(node, parent)) {
        // console.log('before: ', node.children.map(n=>n.type))
        var i = parent.children.findIndex(function (c) { return c === node; });
        if (i !== -1) {
            count.n++;
            (_a = parent.children).splice.apply(_a, [i, 1].concat(node.children));
        }
        // node.children.forEach(c=>removeRedundantNode_(c, info, node, count))
        // console.log('after: ', node.children.map(n=>n.type))
    }
    // else {
    node.children.forEach(function (c) { return removeRedundantNode_(c, info, node, count); });
    // } 
    return count.n;
}
function getParserForLanguage(language) {
    if (language === 'c') {
        return {
            Lexer: require('./grammar/c/CLexer').CLexer,
            Parser: require('./grammar/c/CParser').CParser,
            mainRule: 'compilationUnit'
        };
    }
    else if (language === 'golang') {
        return {
            Lexer: require('./grammar/golang/GolangLexer').GolangLexer,
            Parser: require('./grammar/golang/GolangParser').GolangParser,
            mainRule: 'sourceFile'
        };
    }
    else if (language === 'scala') {
        return {
            Lexer: require('./grammar/scala/ScalaLexer').ScalaLexer,
            Parser: require('./grammar/scala/ScalaParser').ScalaParser,
            mainRule: 'compilationUnit'
        };
    }
    else if (language === 'ruby') {
        return {
            Lexer: require('./grammar/ruby/RubyLexer').RubyLexer,
            Parser: require('./grammar/ruby/RubyParser').RubyParser,
            mainRule: 'prog'
        };
    }
    else if (language === 'java9') {
        return {
            Lexer: require('./grammar/java9/Java9Lexer').Java9Lexer,
            Parser: require('./grammar/java9/Java9Parser').Java9Parser,
            mainRule: 'compilationUnit'
        };
    }
    else if (language === 'lua') {
        return {
            Lexer: require('./grammar/lua/LuaLexer').LuaLexer,
            Parser: require('./grammar/lua/LuaParser').LuaParser,
            mainRule: 'chunk'
        };
    }
    else if (language === 'python3') {
        return {
            Lexer: require('./grammar/python3/Python3Lexer').Python3Lexer,
            Parser: require('./grammar/python3/Python3Parser').Python3Parser,
            mainRule: 'file_input',
            redundantTypes: function (node, parent) { return (node.type.endsWith('expr') || node.type.endsWith('test')
                || ['term', 'factor', 'power', 'small_stmt', 'simple_stmt', 'compound_stmt', 'suite', 'number'].includes(node.type))
                && (node.children.length === 1 ? (node.children[0].text === node.text) : node.children.length === 0) && !(!!parent && parent.text !== node.text); }
        };
    }
    else if (language === 'erlang') {
        return {
            Lexer: require('./grammar/erlang/ErlangLexer').ErlangLexer,
            Parser: require('./grammar/erlang/ErlangParser').ErlangParser,
            mainRule: 'forms',
            redundantTypes: function (node) { return node.children.length === 1 && !!node.type.match(/expr[0-9]+/); }
        };
    }
    else {
        throw new Error('Language unknown: ' + language);
    }
}
//# sourceMappingURL=parseAst.js.map